/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */

import type { onRequestHookHandler } from "fastify";

/**
 * Guards API routes except for "/submit/" and the base routes. Requires
 * that the token in the Authorization header matches the API token in the
 * environment variables.
 * @param request - The request payload from the server.
 * @param response - The reply handler from Fastify.
 * @returns A Fastify reply if the request is invalid, otherwise undefined.
 */
export const authHook: onRequestHookHandler = async(request, response) => {
  if (
    request.url.startsWith("/submit")
      || request.url === "/"
      || request.url === "/health"
      || request.url === "/validate-token"
  ) {
    return undefined;
  }

  const token = request.headers.authorization;
  if (token === undefined) {
    return await response.
      status(400).
      send({ error: "API token is required for this request." });
  }
  if (token !== process.env.API_TOKEN) {
    return await response.status(401).send({ error: "Invalid API token." });
  }
  return undefined;
};
